from typing import Dict, List, Tuple, Union

from pydantic import BaseModel


class Info(BaseModel):
    year: str = None
    version: str = None
    description: str = None
    contributor: str = None
    url: str = None
    date_created: str = None


class Image(BaseModel):
    id: int
    file_name: str
    width: int = None
    height: int = None
    license: int = None
    flickr_url: str = None
    coco_url: str = None
    date_captured: str = None


class License(BaseModel):
    id: int
    name: str
    url: str = None


class Category(BaseModel):
    id: int
    name: str
    supercategory: str = None


class Annotation(BaseModel):
    id: int
    image_id: int
    category_id: int
    bbox: Union[
        Tuple[
            Union[int, float], Union[int, float], Union[int, float], Union[int, float]
        ],
        List,
    ]
    segmentation: Union[List[List[Union[int, float]]], Dict]
    score: float = 0.0
    iscrowd: int = 0
    area: float = 0.0

    def bbox_area(self) -> float:
        if self.bbox is None or self.bbox == []:
            raise ValueError("This annotation has no bbox, so it does not have area")
        return self.bbox[2] * self.bbox[3]

    def is_rle(self) -> bool:
        return (
            self.segmentation is not None
            and isinstance(self.segmentation, Dict)
            and self.segmentation != {}
        )

    def is_polygon(self) -> bool:
        return (
            not self.is_rle()
            and self.segmentation is not None
            and isinstance(self.segmentation, List)
            and self.segmentation != []
        )

    def is_rectangle(self) -> bool:
        return (
            not self.is_rle()
            and not self.is_polygon()
            and (
                self.segmentation is None
                or self.segmentation == {}
                or self.segmentation == []
            )
            and not self.bbox == []
        )

    def polygon_to_list_coordinates(self) -> List[List[List[int]]]:
        if not self.is_polygon():
            raise ValueError("This is not a polygon")

        k = 0
        polygons = []
        for polygon in self.segmentation:
            if len(polygon) % 2 != 0:
                raise ValueError(
                    f"The {k} element of this segmentation is not a polygon."
                )
            polygons.append(
                [
                    [int(polygon[k]), int(polygon[k + 1])]
                    for k in range(0, len(polygon), 2)
                ]
            )
            k += 1

        return polygons


class COCOFile(BaseModel):
    info: Info = Info()
    licenses: List[License] = []
    categories: List[Category] = []
    images: List[Image]
    annotations: List[Annotation]



import json
import logging
import os
import xml.etree.ElementTree as ET

import xmltodict

from picsellia_annotations.coco import COCOFile
from picsellia_annotations.exceptions import FileError, ParsingError
from picsellia_annotations.voc import PascalVOCFile

logger = logging.getLogger("picsellia-annotations")


def read_coco_file(file_path: str) -> COCOFile:

    if not os.path.exists(file_path):
        raise FileError("{} was not found".format(file_path))

    logger.debug("Parsing file..")

    try:
        with open(file_path, "r") as f:
            content = json.load(f)
            cocofile = COCOFile(**content)
    except Exception as e:
        raise ParsingError(str(e))

    return cocofile


def write_coco_file(cocofile: COCOFile) -> str:
    try:
        return cocofile.json()
    except Exception as e:  # pragma: no cover
        raise ParsingError(str(e))


def read_pascal_voc_file(
    file_path: str, check_polygon_consistency: bool = False
) -> PascalVOCFile:

    if not os.path.exists(file_path):
        raise FileError("{} was not found".format(file_path))

    logger.debug("Parsing file {}".format(file_path))

    try:
        tree = ET.parse(file_path)
        xmlstr = ET.tostring(tree.getroot(), encoding="utf-8", method="xml")
        content = xmltodict.parse(xmlstr)
        vocfile = PascalVOCFile(**content)

        # This will raises an error if polygon is not consistent
        if check_polygon_consistency:
            vocfile.annotation.object.polygon_to_list_coordinates()

    except Exception as e:
        raise ParsingError(str(e))

    return vocfile


def write_pascal_voc_file(
    voc: PascalVOCFile, output=None, pretty: bool = True, full_document: bool = False
) -> str:
    try:
        xmlstr = xmltodict.unparse(
            voc.dict(), output=output, full_document=full_document, pretty=pretty
        )
    except Exception as e:  # pragma: no cover
        raise ParsingError(str(e))

    return xmlstr



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['picsellia_annotations']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.9,<2.0', 'xmltodict>=0.13.0,<0.14.0']

setup_kwargs = {
    'name': 'picsellia-annotations',
    'version': '0.5.0',
    'description': 'Package with pydantic schemas of COCO files and VOC File and some utils to read those annotations files',
    'long_description': 'None',
    'author': 'Thomas Darget',
    'author_email': 'thomasdarget@hotmail.fr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)



from typing import Dict, List, Optional, Union

from pydantic import BaseModel


class Size(BaseModel):
    width: int
    height: int
    depth: int


class Source(BaseModel):
    database: str


class BndBox(BaseModel):
    xmin: Union[int, float]
    xmax: Union[int, float]
    ymin: Union[int, float]
    ymax: Union[int, float]


class Object(BaseModel):
    name: str
    pose: Optional[str]
    truncated: Optional[int]
    difficult: Optional[int]
    occluded: Optional[int]
    bndbox: Optional[BndBox]
    polygon: Optional[Dict[str, Union[int, float]]]

    def is_rle(self) -> bool:
        return False

    def is_polygon(self) -> bool:
        return self.polygon is not None

    def is_rectangle(self) -> bool:
        return self.polygon is None and self.bndbox is not None

    def polygon_to_list_coordinates(self) -> List[List[int]]:
        if not self.is_polygon():
            raise ValueError("Not a polygon")

        coords = []
        for i in range(1, 1 + len(self.polygon) // 2):
            x = "x" + str(i)
            y = "y" + str(i)
            if x not in self.polygon or y not in self.polygon:
                raise ValueError("{} or {} not found in this polygon.".format(x, y))

            coords.append([int(self.polygon[x]), int(self.polygon[y])])

        return coords


class Annotation(BaseModel):
    filename: str
    object: Union[Object, List[Object]]
    path: Optional[str]
    folder: Optional[str]
    source: Optional[Source]
    size: Optional[Size]
    segmented: Optional[int]


class PascalVOCFile(BaseModel):
    annotation: Annotation
